<?php

namespace Drupal\queue_monitor\Commands;

use Drupal\Core\Queue\QueueFactory;
use Drupal\Core\Queue\QueueWorkerManagerInterface;
use Drush\Commands\DrushCommands;
use Drush\Exceptions\UserAbortException;
use Drush\Utils\StringUtils;

/**
 * Class QueueMonitorCommands
 *
 * @package Drupal\queue_monitor\Commands
 */
class QueueMonitorCommands extends DrushCommands {

  protected $workerManager;

  protected $queueService;

  protected static $queues;

  /**
   * QueueMonitorCommands constructor.
   *
   * @param \Drupal\Core\Queue\QueueWorkerManagerInterface $workerManager
   * @param \Drupal\Core\Queue\QueueFactory                $queueService
   */
  public function __construct(QueueWorkerManagerInterface $workerManager, QueueFactory $queueService) {
    parent::__construct();
    $this->workerManager = $workerManager;
    $this->queueService = $queueService;
  }

  /**
   * get worker manager api.
   *
   * @return \Drupal\Core\Queue\QueueWorkerManagerInterface
   */
  public function getWorkerManager()
  {
    return $this->workerManager;
  }

  /**
   * @return \Drupal\Core\Queue\QueueFactory
   */
  public function getQueueService() {
    return $this->queueService;
  }

  /**
   * {@inheritdoc}
   */
  public function getQueues() {
    static::$queues;
    if (!isset(static::$queues)) {
      static::$queues = [];
      foreach ($this->getWorkerManager()->getDefinitions() as $name => $info) {
        static::$queues[$name] = $info;
      }
    }
    return static::$queues;
  }

  /**
   * {@inheritdoc}
   *
   * @return \Drupal\Core\Queue\QueueInterface
   */
  public function getQueue($name) {
    return $this->getQueueService()->get($name);
  }

  /**
   * Run queue by name.
   *
   * @param       $name
   *
   * @command queue_monitor:run
   * @usage   queue_monitor:run myqueue
   *
   * @throws \Exception
   */
  public function queueRun($name) {
    while (TRUE) {
      $worker = $this->getWorkerManager()->createInstance($name);
      $queue = $this->getQueue($name);
      while ($item = $queue->claimItem()) {
        try {

          $this->logger()->info(dt('Processing item @id from @name queue.', [
            '@name' => $name,
            '@id' => $item->item_id,
          ]));
          $worker->processItem($item->data);
          $queue->deleteItem($item);

        } catch (RequeueException $e) {

          // The worker requested the task to be immediately requeued.
          $queue->releaseItem($item);

        } catch (SuspendQueueException $e) {

          // If the worker indicates there is a problem with the whole queue,
          // release the item.
          $queue->releaseItem($item);
          throw new \Exception($e->getMessage());

        }
      }
      $config = \Drupal::config('queue_monitor.settings');
      sleep($config->get('sleep'));
    }
  }

  /**
   * Run all queue.
   *
   * @command queue_monitor:runall
   * @usage   queue_monitor:runall
   *
   * @throws \Exception
   */
  public function queueRunAll() {
    while (TRUE) {
      foreach (array_keys($this->getQueues()) as $name) {
        $worker = $this->getWorkerManager()->createInstance($name);
        $queue = $this->getQueue($name);
        while ($item = $queue->claimItem()) {
          try {

            $this->logger()->info(dt('Processing item @id from @name queue.', ['@name' => $name, '@id' => $item->item_id]));
            $worker->processItem($item->data);
            $queue->deleteItem($item);

          } catch (RequeueException $e) {

            // The worker requested the task to be immediately requeued.
            $queue->releaseItem($item);

          } catch (SuspendQueueException $e) {

            // If the worker indicates there is a problem with the whole queue,
            // release the item.
            $queue->releaseItem($item);

            throw new \Exception($e->getMessage());
          }
        }
      }

      $config = \Drupal::config('queue_monitor.settings');
      sleep($config->get('sleep'));
    }
  }

}
